unit uVoigt;

interface

Type
  TVoigt = procedure   (a,V:real;var H,F:real);

procedure VoigtTaylor  (a,V:real;var H,F:real);
procedure VoigtHumlicek(a,V:real;var H,F:real);
procedure VoigtSpline  (a,V:real;var H,F:real);
procedure VoigtIntegral(a,V:real;var H,F:real);

procedure SetVoigt(sMethod:string);

var
  VOIGT        : TVoigt;
  sVoigtMethod : string;

implementation

uses Windows,SysUtils,
     uTypes,uFMM;

(* ,       *)
(*                                   *)
var

  (*     V  a*a   *)
  (* VoigtHIntegrand  VoigtFIntegrand                       *)
  global_v, global_a2      : Extended;

  (*     ? *)
  global_QSetSplineArray   : boolean;

  (*========       "a" =======*)
  global_Na                : integer;  (*    "a" *)
  global_lamin, global_dla : extended; (* .   "a"        *)

  (*=  2-       "V" =*)
  global_NV1,   global_NV2 : integer;  (*  2-  "V"  *)
  global_Vmin,  global_dV  : extended; (* ..    1     *)
  global_V12               : extended; (* = 4   1  2    *)
  global_lVmin, global_dlV : extended; (* ..  -  log(V) *)

  global_EPS, global_boundV : extended;

(* ,       a  V *)

  Arr_a    : TARe; (*   "a"    *)
  Arr_la   : TARe; (*  log(a) *)
  Arr_V1   : TARe; (*   "V"   1 *)
  Arr_V2   : TARe; (*   "V"   2 *)
  Arr_lV   : TARe; (*  log(V)   2        *)

(*   -   H  F    *)

  Arr_H1  : TARe2; (*  H   1 *)
  Arr_H2  : TARe2; (*  H   2 *)
  Arr_F1  : TARe2; (*  F   1 *)
  Arr_F2  : TARe2; (*  F   2 *)

(*   - H  F    *)
(*    V     a *)

  Arr_BH1  : TARe2;
  Arr_CH1  : TARe2;
  Arr_DH1  : TARe2;
  Arr_BH2  : TARe2;
  Arr_CH2  : TARe2;
  Arr_DH2  : TARe2;
  Arr_BF1  : TARe2;
  Arr_CF1  : TARe2;
  Arr_DF1  : TARe2;
  Arr_BF2  : TARe2;
  Arr_CF2  : TARe2;
  Arr_DF2  : TARe2;



(*============================================================================*)

(*================================================*)
(*     H  F *)
(*================================================*)


function VoigtHIntegrand(x:Extended):Extended;
var ey2,dv,dv2,z:Extended;
begin
  dv  := (global_v-x);
  dv2 := dv*dv;
  z   := dv2+global_a2;
  ey2 := exp(-(x*x));
  result := ey2/z;
end;

function VoigtFIntegrand(x:Extended):Extended;
var ey2,dv,dv2,z:Extended;
begin
  dv  := (global_v-x);
  dv2 := dv*dv;
  z   := dv2+global_a2;
  ey2 := exp(-(x*x));
  result := ey2*dv/z;
end;


{
type
  TFunc = function(a2,v,x:real):real;
}


(*==============================================================*)
(*   H   F      *)
(*      QUANC8        *)
(*   .     *)
(*   Delphi,     *)
(*     "a"                            *)
(*==============================================================*)

function VoigtHIntegral(a,v:Extended):Extended;
var
  func  : TFuncEX;
  AbsErr,RelErr : Extended;
  Err   : Extended; (*    *)
  S     : Extended; (*  () *)
  FLAG  : Extended; (* 0  QUANC8    *)
  NOFUN : integer;  (*   func *)
begin
  func := VoigtHIntegrand;
  global_a2 := a*a;
  global_v  := v;
  AbsErr := 0;
  RelErr := global_EPS;
  QUANC8(func,-global_boundV,global_boundV,AbsErr,RELERR,S,Err,NOFUN,FLAG);
  result := a*S/PI;
end;

function VoigtFIntegral(a,v:Extended):Extended;
var
  func  : TFuncEX;
  AbsErr,RelErr : Extended;
  Err   : Extended; (*    *)
  S     : Extended;
  FLAG  : Extended;
  NOFUN : integer;  (*    func *)
begin
  if abs(v) < 1E-7 then begin result := 0; Exit end;
  func := VoigtFIntegrand;
  global_a2 := a*a;
  global_v  := v;
  AbsErr := 0;
  RelErr := global_EPS;
  QUANC8(func,-global_boundV,global_boundV,AbsErr,RELERR,S,Err,NOFUN,FLAG);
  result := S/(2*PI);
end;


(*==============================================================*)
(*           a *)
(*==============================================================*)

(*************************************************************************
  F(X).

 :
    X   -    

 :
    X   -   

 :

                         
 -10,10            10000     6.9e-16      1.0e-16

Cephes Math Library Release 2.8:  June, 2000
Copyright 1984, 1987, 1989, 2000 by Stephen L. Moshier
*************************************************************************)
function DawsonIntegral(X : Double):Double;
var
    X2 : Double;
    Y  : Double;
    Sg : Integer;
    AN : Double;
    AD : Double;
    BN : Double;
    BD : Double;
    CN : Double;
    CD : Double;
begin
    Sg := 1;
    if X<0 then begin Sg := -1; X := -X end;
    if X<3.25 then
    begin
        x2 := x*x;
        AN := 1.13681498971755972054E-11;
        AN := AN*x2+8.49262267667473811108E-10;
        AN := AN*x2+1.94434204175553054283E-8;
        AN := AN*x2+9.53151741254484363489E-7;
        AN := AN*x2+3.07828309874913200438E-6;
        AN := AN*x2+3.52513368520288738649E-4;
        AN := AN*x2+-8.50149846724410912031E-4;
        AN := AN*x2+4.22618223005546594270E-2;
        AN := AN*x2+-9.17480371773452345351E-2;
        AN := AN*x2+9.99999999999999994612E-1;
        AD := 2.40372073066762605484E-11;
        AD := AD*x2+1.48864681368493396752E-9;
        AD := AD*x2+5.21265281010541664570E-8;
        AD := AD*x2+1.27258478273186970203E-6;
        AD := AD*x2+2.32490249820789513991E-5;
        AD := AD*x2+3.25524741826057911661E-4;
        AD := AD*x2+3.48805814657162590916E-3;
        AD := AD*x2+2.79448531198828973716E-2;
        AD := AD*x2+1.58874241960120565368E-1;
        AD := AD*x2+5.74918629489320327824E-1;
        AD := AD*x2+1.00000000000000000539E0;
        y := x*AN/AD;
        Result := Sg*Y;
        Exit;
    end;
    x2 := 1.0/(x*x);
    if x<6.25 then
    begin
        BN := 5.08955156417900903354E-1;
        BN := BN*x2-2.44754418142697847934E-1;
        BN := BN*x2+9.41512335303534411857E-2;
        BN := BN*x2-2.18711255142039025206E-2;
        BN := BN*x2+3.66207612329569181322E-3;
        BN := BN*x2-4.23209114460388756528E-4;
        BN := BN*x2+3.59641304793896631888E-5;
        BN := BN*x2-2.14640351719968974225E-6;
        BN := BN*x2+9.10010780076391431042E-8;
        BN := BN*x2-2.40274520828250956942E-9;
        BN := BN*x2+3.59233385440928410398E-11;
        BD := 1.00000000000000000000E0;
        BD := BD*x2-6.31839869873368190192E-1;
        BD := BD*x2+2.36706788228248691528E-1;
        BD := BD*x2-5.31806367003223277662E-2;
        BD := BD*x2+8.48041718586295374409E-3;
        BD := BD*x2-9.47996768486665330168E-4;
        BD := BD*x2+7.81025592944552338085E-5;
        BD := BD*x2-4.55875153252442634831E-6;
        BD := BD*x2+1.89100358111421846170E-7;
        BD := BD*x2-4.91324691331920606875E-9;
        BD := BD*x2+7.18466403235734541950E-11;
        y := 1.0/x+x2*BN/(BD*x);
        Result := sg*0.5*y;
        Exit;
    end;
    if x>1.0E9 then
    begin
        Result := sg*0.5/x;
        Exit;
    end;
    CN := -5.90592860534773254987E-1;
    CN := CN*x2+6.29235242724368800674E-1;
    CN := CN*x2-1.72858975380388136411E-1;
    CN := CN*x2+1.64837047825189632310E-2;
    CN := CN*x2-4.86827613020462700845E-4;
    CD := 1.00000000000000000000E0;
    CD := CD*x2-2.69820057197544900361E0;
    CD := CD*x2+1.73270799045947845857E0;
    CD := CD*x2-3.93708582281939493482E-1;
    CD := CD*x2+3.44278924041233391079E-2;
    CD := CD*x2-9.73655226040941223894E-4;
    y := 1.0/x+x2*CN/(CD*x);
    Result := sg*0.5*y;
end;

(*  H  F        *)
procedure VoigtTaylor(a,v:real;var h,f:real);
const sp = 0.564189583547756280; (* 1 /     *)
var v2,ev,fv : real;
    a2,a3    : real;
    mv       : real;
begin
  v2 := v*v;
  ev := exp(-v2);
  fv := DawsonIntegral(v);
  mv := 1-2*v2;                         (* 1 - 2v^2 *)
  a2 := a*a;
  a3 := a2*a;
  h  :=        ev                             (* H0 *)
        + a * (-2*sp)*(1-2*v*fv)              (* H1 *)
        + a2* mv*ev                           (* H2 *)
        + a3* (-4/3*sp)*(1-v2-(3-2*v2)*v*fv)  (* H3 *)
//      + a4* (3-4*v2*(3+v2))*ev/6            (* H4 *)
        ;
  f :=  +      fv*sp                          (* F0 *)
        + a * (-v*ev)                         (* F1 *)
        + a2* (v + fv*mv)*sp                  (* F2 *)
        + a3* (-v*ev)*(2+mv)/3;               (* F3 *)
  (*  f      1969 *)
  (* Beckers, J.M., Solar Physics, 1969. v.9, p.372.*)
end; (* VoigtTaylor *)


PROCEDURE VoigtHumlicek(a,V:REAL;VAR H,F:REAL);
(* Humlicek J., JQSRT, 1979. V.21. p.309. *)
CONST
  T : ARRAY [1..6] OF REAL =
           (  0.314240376,
              0.947788391,
              1.59768264,
              2.27950708,
              3.02063703,
              3.8897249);
  C : ARRAY [1..6] OF REAL =
           (  1.01172805,
             -0.75197147,
              1.2557727E-2,
              1.00220082E-2,
             -2.42068135E-4,
              5.00848061E-7);
  S : ARRAY [1..6] OF REAL =
           (  1.393237,
              0.231152406,
             -0.155351466,
              6.21836624E-3,
              9.19082986E-5,
             -6.27525958E-7);
VAR
  R,R2,Y1,Y2,Y3 : REAL;
  D,D1,D2,D3,D4 : REAL;
              I : INTEGER;
           ISig : integer;
BEGIN
  if a < 0 then begin ISig := -1; a := Abs(a) end else ISig := 1;
  H :=0.0;
  F :=0.0;
  Y1:=a+1.5;
  Y2:=Y1*Y1;
  IF (a>0.85) OR ((ABS(V))<(18.1*a+1.65))
    THEN BEGIN
(** REGION I **)
      FOR I:=1 TO 6 DO BEGIN
        R := V-T[I];
        D := 1/(R*R+Y2);
        D1:= Y1*D;
        D2:= R*D;
        R := V+T[I];
        D := 1/(R*R+Y2);
        D3:= Y1*D;
        D4:= R*D;
        H := H+C[I]*(D1+D3)-S[I]*(D2-D4);
        F := F+C[I]*(D2+D4)+S[I]*(D1-D3);
      END;
      F:=-0.5*F*ISig;
    END
    ELSE BEGIN
(** REGION II **)
      IF (ABS(V)<12.0) THEN H:=EXP(-V*V);
      Y3:=a+3.0;
      FOR I:=1 TO 6 DO BEGIN
        R := V-T[I];
        R2:= R*R;
        D := 1/(R2+Y2);
        D1:= Y1*D;
        D2:= R*D;
        H := H+a*(C[I]*(R*D2-1.5*D1)+S[I]*Y3*D2)/(R2+2.25);
        R := V+T[I];
        R2:= R*R;
        D := 1/(R2+Y2);
        D3:= Y1*D;
        D4:= R*D;
        H := H+a*(C[I]*(R*D4-1.5*D3)-S[I]*Y3*D4)/(R2+2.25);
        F := F+C[I]*(D2+D4)+S[I]*(D1-D3);
      END;
      F:=-0.5*F*ISig;
    END;(* ELSE *)
  END;(* VoigtHumlicek *)

(*=========================================================*)
(*       *)
(* C  "a"      *)
(* C  "V"    :        *)
(*   1.           (0 < V < 4)     *)
(*   2.      ( V > 4)     *)
(*    "a" < 10E-3        *)
(*        "a"   *)
(*             .. (C) 2015                   *)
(*=========================================================*)
procedure VoigtSpline(a,V:real; var H,F:real);
var la,lV : real;
  J,JJ,I,K,M : integer;
  AA4,AH4,BH4,CH4,DH4 : TAR4;
      AF4,BF4,CF4,DF4 : TAR4;
  dla,dla1 : real;
  RF : real;
  H0,H1,H2,H3,H4,F0,F1,F2,F3 : real;
begin
  if a < 1E-3 then begin VoigtTaylor(a,V,H,F); Exit end;

  if V < 0 then begin RF:=-1; V := -V end  (*  F  *)
           else begin RF:= 1;         end; (*   "V" *)

  la := system.ln(a);(*     ,   *)
  J := 1 + trunc((la - global_lamin)/global_dla);

  if V <= Global_V12 then begin (*     V *)
    I := 1 + trunc((V  - global_Vmin)/global_dV );
    JJ := J-2;
    for K := 1 to 4 do begin
      J := JJ+K;
      AH4[K] := SEVAL_I(I,V,Arr_V1,Arr_H1[J],Arr_BH1[J],Arr_CH1[J],Arr_DH1[J]);
      AF4[K] := SEVAL_I(I,V,Arr_V1,Arr_F1[J],Arr_BF1[J],Arr_CF1[J],Arr_DF1[J]);
      AA4[K] := Arr_la[J];
    end;
  end
  else begin (*       *)
    lV := system.ln(V);
    I  := 1 + trunc((lV  - global_lVmin)/global_dlV );
    JJ := J-2;
    for K := 1 to 4 do begin
      J := JJ+K;
      AH4[K]:= SEVAL_I(I,lV,Arr_lV,Arr_H2[J],Arr_BH2[J],Arr_CH2[J],Arr_DH2[J]);
      AF4[K]:= SEVAL_I(I,lV,Arr_lV,Arr_F2[J],Arr_BF2[J],Arr_CF2[J],Arr_DF2[J]);
      AA4[K]:= Arr_la[J];
    end;
  end;

  (*    "a"  4-   AH4 *)
  uFMM.SPLINE4(AA4,AH4,BH4,CH4,DH4);
  uFMM.SPLINE4(AA4,AF4,BF4,CF4,DF4);

  dla:= la-AA4[2]; dla1 := AA4[3]-la;
  if dla < dla1 then M := 2
                else begin M := 3; dla := -dla1 end;
  H  := AH4[M]+dla*(BH4[M]+dla*(CH4[M]+dla*DH4[M]));
  if V < 1E-8 then F := 0
              else F := RF*(AF4[M]+dla*(BF4[M]+dla*(CF4[M]+dla*DF4[M])));
end; (* VoigtSpline *)




(*============================================================================*)
(*  H  F        *)
(*    QUANC8                                                 *)
(*                                                                            *)
(* VoigtIntegral  ,      *)
(*                                                                            *)
(*                           *)
(*   global_EPS  global_boundV    QUANC8                *)
(*============================================================================*)
procedure VoigtIntegral(a,V:real; var H,F:real);
begin
  H := VoigtHIntegral(a,V);
  F := VoigtFIntegral(a,V);
end; (* VoigtIntegral *)


(*========================================================================*)
(*         *)
(*========================================================================*)

(*  ,      *)
procedure SetGlobal(EPS,boundV,
                    dV1,V12,Vmax,
                    amin,amax:real;
                    Na,NV1,NV2:integer);
var lVmax,lVmin,lamax,Vmin : extended;
  na_rim, nV_rim : integer;
begin

  (*         *)
  (*        ,   *)
  (*           *)
  (*    "a"  "V"                        *)
  nV_rim := 5;      (* 5 > 2, ..     *)
  na_rim := 5;

  (*   QUANC8 *)
  global_EPS    := EPS;
  global_boundV := boundV;

  global_Na    := Na;  (*      "a" *)
  global_NV1   := NV1; (*      "V"    *)
  global_NV2   := NV2; (*    - "V"   *)

  Vmin         := 0;   (* H/F   V    0 *)
  global_dV    := dV1;
  global_Vmin  := Vmin - nV_rim*global_dV;

  global_V12   := V12; (*   1/2 = /   *)

  lVmin        := system.ln(V12);  (* .  .- *)
  lVmax        := system.ln(VMax); (* . .- *)
  global_dlV   := (lVmax-lVmin) / (NV2-1-2*nV_rim);
  global_lVmin := lVmin - nV_rim*global_dlV;

  global_lamin := system.ln(amin);
         lamax := system.ln(amax);
  global_dla   := (lamax-global_lamin) / (global_Na-1-2*na_rim);
  global_lamin := global_lamin - na_rim*global_dla;
//       lamax :=        lamax + na_rim*global_dla;
end; (* SetGlobal *)

procedure Arr_GetMem;
begin
  (*=======================================================================*)
  (*                                            *)
  (*=======================================================================*)

  SetLength(Arr_V1, global_NV1+1);  (*  1 :        V  *)
  SetLength(Arr_V2, global_NV2+1);  (*  2 :        V  *)
  SetLength(Arr_lV, global_NV2+1);  (*  2 :    log(V) *)

  SetLength(Arr_a, global_Na+1);    (*  1  2      a  *)
  SetLength(Arr_la,global_Na+1);    (*  1  2  log(a) *)

  (*  1,   H  : *)

  SetLength(Arr_H1  ,global_Na+1,global_NV1+1); (*   H *)
  SetLength(Arr_BH1 ,global_Na+1,global_NV1+1); (*                 *)
  SetLength(Arr_CH1 ,global_Na+1,global_NV1+1); (*              *)
  SetLength(Arr_DH1 ,global_Na+1,global_NV1+1); (*      H     *)

  (*  2,   H  : *)

  SetLength(Arr_H2  ,global_Na+1,global_NV2+1); (*   H *)
  SetLength(Arr_BH2 ,global_Na+1,global_NV2+1); (*                 *)
  SetLength(Arr_CH2 ,global_Na+1,global_NV2+1); (*              *)
  SetLength(Arr_DH2 ,global_Na+1,global_NV2+1); (*      H     *)

  (*  1,   F  : *)

  SetLength(Arr_F1  ,global_Na+1,global_NV1+1); (*   F *)
  SetLength(Arr_BF1 ,global_Na+1,global_NV1+1); (*                 *)
  SetLength(Arr_CF1 ,global_Na+1,global_NV1+1); (*              *)
  SetLength(Arr_DF1 ,global_Na+1,global_NV1+1); (*      F     *)

  (*  2,   F  : *)

  SetLength(Arr_F2  ,global_Na+1,global_NV2+1); (*   F *)
  SetLength(Arr_BF2 ,global_Na+1,global_NV2+1); (*                 *)
  SetLength(Arr_CF2 ,global_Na+1,global_NV2+1); (*              *)
  SetLength(Arr_DF2 ,global_Na+1,global_NV2+1); (*      F     *)

end; (* Arr_GetMem *)


procedure Fill_aV_arr;
var I,J : integer; V,V2,lV,la : real;
begin
  (*======================================================================*)
  (*        a    V                                       *)
  (*======================================================================*)

    (*======================*)
    (*  a          *)
    (*    a         *)
    (*======================*)

  for J := 1 to global_Na do begin
    la := global_lamin + (J-1)*global_dla;
    Arr_la[J] := la;
    Arr_a[J]  := system.EXP(la);
  end;

    (*======================*)
    (*  V           *)
    (*     1        *)
    (*======================*)

  for I := 1 to global_NV1 do begin
    V  := global_Vmin  + (I-1)*global_dV ; Arr_V1[I] := V;
  end;

    (*======================*)
    (*  V          *)
    (*    V         *)
    (*     2        *)
    (*======================*)

  for I := 1 to global_NV2 do begin
    lV := global_lVmin + (I-1)*global_dlV; Arr_lV[I] := lV;
    V2 := exp(lV);                         Arr_V2[I] := V2;
  end;
end; (* Fill_aV_arr *)

(*   H     *)
procedure Calc_H_Arr;
var I,J : integer; a,V,V2,H : real;
begin
  for I := 1 to global_NV1 do begin
    for J := 1 to global_Na do begin
      a  := Arr_a[J];
      V  := Arr_V1[I];
      H  := VoigtHIntegral(a,V);
      Arr_H1[J,I] := H;
    end;
  end;

  for I := 1 to global_NV2 do begin
    for J := 1 to global_Na do begin
      a  := Arr_a[J];
      V2 := Arr_V2[I];
      H  := VoigtHIntegral(a,V2);
      Arr_H2[J,I] := H;
    end;
  end;
end;

(*   F     *)
procedure Calc_F_Arr;
var I,J : integer; a,V,V2,F : real;
begin
  for I := 1 to global_NV1 do begin
    for J := 1 to global_Na do begin
      a  := Arr_a[J];
      V  := Arr_V1[I];
      F  := VoigtFIntegral(a,V);
      Arr_F1[J,I] := F;
    end;
  end;

  for I := 1 to global_NV2 do begin
    for J := 1 to global_Na do begin
      a  := Arr_a[J];
      V2 := Arr_V2[I];
      F  := VoigtFIntegral(a,V2);
      Arr_F2[J,I] := F;
    end;
  end;
end;


procedure FillArr_H(Nam,Nam2:string;QCalc:boolean);
var J : integer;
begin
(*=============================================================*)
(*       (QCalc=true),                         *)
(*      H          *)
(*=============================================================*)
  if QCalc then begin
    Calc_H_Arr;
    (*   Arr_H1  Arr_H2    *)
    WrTARe2(Arr_H1,global_Na+1,global_NV1+1,Nam );
    WrTARe2(Arr_H2,global_Na+1,global_NV2+1,Nam2);
  end; (* if QCalc *)

  RdTARe2(Arr_H1,global_Na+1,global_NV1+1,Nam );
  RdTARe2(Arr_H2,global_Na+1,global_NV2+1,Nam2);

  (*      V   a *)
  for J := 1 to global_Na do begin
    uFMM.SPLINE(Arr_V1,Arr_H1[J], Arr_BH1[J], Arr_CH1[J], Arr_DH1[J]);
    uFMM.SPLINE(Arr_lV,Arr_H2[J], Arr_BH2[J], Arr_CH2[J], Arr_DH2[J]);
  end;
end;

procedure FillArr_F(Nam,Nam2:string;QCalc:boolean);
var J : integer;
begin
(*=============================================================*)
(*       (QCalc=true),                         *)
(*      F          *)
(*=============================================================*)
  if QCalc then begin
    Calc_F_Arr;
    (*   Arr_F1  Arr_F2    *)
    WrTARe2(Arr_F1,global_Na+1,global_NV1+1,Nam );
    WrTARe2(Arr_F2,global_Na+1,global_NV2+1,Nam2);
  end;

  RdTARe2(Arr_F1,global_Na+1,global_NV1+1,Nam );
  RdTARe2(Arr_F2,global_Na+1,global_NV2+1,Nam2);

  (*      V   a *)
  for J := 1 to global_Na do begin
    uFMM.SPLINE(Arr_V1,Arr_F1[J], Arr_BF1[J], Arr_CF1[J], Arr_DF1[J]);
    uFMM.SPLINE(Arr_lV,Arr_F2[J], Arr_BF2[J], Arr_CF2[J], Arr_DF2[J]);
  end;
end;


(*===================================================*)
(*                      *)
(*  H  F            *)
(*  a  V     *)
(*                   *)
(*===================================================*)
procedure LoadVoigtSplineArray;
var NamBeg,NamEnd,fNameH1,fNameF1,fNameH2,fNameF2 : string;
    QExist,QCalc : boolean;
begin
  (*     *)
  NamBeg := 'V1350a2000_';
  NamEnd := '.dat';

  fNameH1 := NamBeg+'H1'+NamEnd;
  fNameH2 := NamBeg+'H2'+NamEnd;
  fNameF1 := NamBeg+'F1'+NamEnd;
  fNameF2 := NamBeg+'F2'+NamEnd;
  Arr_GetMem;(*      (  )*)
  Fill_aV_arr;(*     a    V                           *)

  QExist := (SysUtils.FileExists(fNameH1) and
             SysUtils.FileExists(fNameH2) and
             SysUtils.FileExists(fNameF1) and
             SysUtils.FileExists(fNameF2));
  QCalc := Not QExist;

  if QCalc then begin
    if Not QUERY('     '
                 +#13#10+' 84    .'
                 +#13#10
                 +#13#10+'  "" -   !')
    then begin
      VOIGT := VoigtHumlicek;  (*  VoigtHumlicek ,  *)
                               (*       *)
      sVoigtMethod := 'Humlicek';
      global_QSetSplineArray := false;
      Exit;
    end;
  end;

  FillArr_H(fNameH1,fNameH2,QCalc);
  FillArr_F(fNameF1,fNameF2,QCalc);
  global_QSetSplineArray := true;
  VOIGT := VoigtSpline;  (*  VoigtSpline ,    *)
                         (*       *)
  sVoigtMethod := 'Spline';
end; (* LoadVoigtSplineArray *)



(*  ,      -  *)
procedure SetVoigt(sMethod:string);
begin
  if sMethod = 'Taylor'
  then VOIGT := VoigtTaylor
  else
  if sMethod = 'Integral'
  then VOIGT := VoigtIntegral
  else
  if sMethod = 'Spline' then begin
    if global_QSetSplineArray
    then VOIGT := VoigtSpline
    else begin
       LoadVoigtSplineArray;(*       *)
                           (*       *)
{
      WarnAbs(
 '         ,'+
 #13#10 + '      '+#13#10+
          '  !');
      sMethod := 'Humlicek';
      VOIGT := VoigtHumlicek;
}
    end;
  end
  else
  if sMethod = 'Humlicek'
  then VOIGT := VoigtHumlicek
  else begin
    WarnAbs(
 '       <'+sMethod+'>!'+
 #13#10 + ' : Taylor, Integral, Spline  Humlicek.'+#13#10+
          '  !');
    sMethod := 'Humlicek';
    VOIGT := VoigtHumlicek;
  end;
  sVoigtMethod := sMethod;
end; (* SetVoigt *)


(*===================================================*)
(*   ,      *)
(*   :                     *)
(* ~10E-12                         *)
(*===================================================*)

procedure DefineGlobal;
var EPS,boundV,VV,Vmax,amin,amax,dV : real;
    NV,NV1,NV2,Na : integer;
begin
  EPS    := 5E-14; (* ,    QUANC8               *)
  boundV := 2000;  (*    V  QUANC8                *)

  NV     := 1350;  (*     V      *)
  NV1    := NV;
  NV2    := NV;
  Na     := 2000;  (*      "a"                *)

  dV     := 0.003; (*      V                           *)
  VV     :=  4;    (*        V         *)
//Vmin   :=  0;    (*    V                           *)
  Vmax   := 2000;  (*   V                                     *)
  amin   := 1E-3;  (*    "a"                                   *)
  amax   := 1E+2;  (*   "a"                                   *)

  SetGlobal(EPS,boundV,dV,VV,Vmax,amin,amax,Na,NV1,NV2);
end;


(* ,     UNIT uVoigt *)
begin
  global_QSetSplineArray := false;
  DefineGlobal;        (*     *)
  SetVoigt('Humlicek');
(*       /     *)
(*           "Spline"                *)
(*           :     *)
//LoadVoigtSplineArray;(*       *)
                       (*       *)
end.
